package gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;

/**
 * This code is provided for use in the course SWE-622, Distributed Software
 * Systems. Feel free to use and change this code, but the author is not liable
 * for any such use or misuse.
 * 
 * This class provides the interface IOperationsConsole and requires ITrader.
 * Create an instance of this class to show a window for a trader console.
 * 
 * @see WindowClosingListener
 * 
 * @author jpsousa
 * @date Sep 21, 2006
 */
public class JFrameTraderGUI extends JFrame implements IOperationsConsole {
	private static final long serialVersionUID = 1L;

	private ITrader _trader;

	private JScrollOperationsTable _operationsTable;

	/**
	 * Shows a window for a trader console.
	 * @param trader
	 *            will receive callbacks, as defined in ITrader
	 * @param trName
	 *            the name for the trader to be displayed in the window title
	 */
	public JFrameTraderGUI(ITrader trader, String trName) {
		super("Trader: " + trName);
		_trader = trader;
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			e.printStackTrace();
		}
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowClosingListener(this));

		_operationsTable = new JScrollOperationsTable();
		JPanelTraderConsole panel = new JPanelTraderConsole(_operationsTable);
		setContentPane(panel);
		pack();
		setVisible(true);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gui.IOperationsConsole#logOperation(java.lang.String,
	 *      java.lang.String, java.lang.String, java.lang.String,
	 *      java.lang.String)
	 */
	public void logOperation(String exchange, String op, String product,
			String qt, String price, boolean stale) {
		_operationsTable.logOperation(exchange, op, product, qt, price, stale);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see gui.IOperationsConsole#clearLog()
	 */
	public void clearLog() {
		_operationsTable.clearLog();
	}

	/**
	 * @author jpsousa
	 * @date Sep 21, 2006
	 */
	private class JPanelTraderConsole extends JPanel implements ActionListener {
		private static final long serialVersionUID = 1L;

		private static final String __BLANK_FIELD_MSG = "Fill in the relevant fields.";

		private JTextField _exchange = new JTextField();

		private JTextField _product = new JTextField();

		private JTextField _price = new JTextField();

		private JTextField _qt = new JTextField();

		private JPanelTraderConsole(JScrollOperationsTable opsTable) {
			setLayout(new GridBagLayout());
			GridBagConstraints gbConstraints = new GridBagConstraints();

			gbConstraints.gridy = 0;
			gbConstraints.fill = GridBagConstraints.HORIZONTAL;
			// Exchange label
			gbConstraints.gridx = 0;
			gbConstraints.weightx = 0;
			gbConstraints.insets = new Insets(5, 10, 0, 0);
			add(new JLabel("Exchange"), gbConstraints);
			// Exchange field
			gbConstraints.gridx = 1;
			gbConstraints.weightx = 1;
			gbConstraints.insets = new Insets(5, 5, 0, 0);
			add(_exchange, gbConstraints);
			// Product label
			gbConstraints.gridx = 2;
			gbConstraints.weightx = 0;
			gbConstraints.insets = new Insets(5, 10, 0, 0);
			add(new JLabel("Product"), gbConstraints);
			// Product field
			gbConstraints.gridx = 3;
			gbConstraints.weightx = 1;
			gbConstraints.insets = new Insets(5, 5, 0, 10);
			add(_product, gbConstraints);

			gbConstraints.gridx = 0;
			// Price label
			gbConstraints.gridy = 1;
			gbConstraints.weightx = 0;
			gbConstraints.insets = new Insets(10, 10, 0, 0);
			add(new JLabel("Price"), gbConstraints);
			// Price field
			gbConstraints.gridy = 2;
			gbConstraints.weightx = 1;
			gbConstraints.insets = new Insets(5, 10, 0, 0);
			add(_price, gbConstraints);

			gbConstraints.gridx = 2;
			// Qt label
			gbConstraints.gridy = 1;
			gbConstraints.weightx = 0;
			gbConstraints.insets = new Insets(10, 10, 0, 0);
			add(new JLabel("Qt"), gbConstraints);
			// Qt field
			gbConstraints.gridy = 2;
			gbConstraints.weightx = 1;
			gbConstraints.insets = new Insets(5, 10, 0, 0);
			add(_qt, gbConstraints);

			gbConstraints.gridx = 1;
			gbConstraints.weightx = 0;
			gbConstraints.fill = GridBagConstraints.BOTH;
			// Sell button
			JButton sellBt = new JButton();
			sellBt.setText(IOperationsConsole.__SELL);
			sellBt.setActionCommand(IOperationsConsole.__SELL);
			sellBt.addActionListener(this);
			gbConstraints.gridy = 1;
			gbConstraints.gridheight = 2;
			gbConstraints.insets = new Insets(10, 5, 0, 0);
			add(sellBt, gbConstraints);

			gbConstraints.gridx = 3;
			// Buy button
			JButton buyBt = new JButton();
			buyBt.setText(IOperationsConsole.__BUY);
			buyBt.setActionCommand(IOperationsConsole.__BUY);
			buyBt.addActionListener(this);
			gbConstraints.gridy = 1;
			gbConstraints.gridheight = 2;
			gbConstraints.insets = new Insets(10, 5, 0, 10);
			add(buyBt, gbConstraints);

			// Operations table
			gbConstraints.gridy = 10;
			gbConstraints.gridx = 0;
			gbConstraints.gridwidth = 4;
			gbConstraints.weightx = 1;
			gbConstraints.weighty = 1;
			gbConstraints.insets = new Insets(10, 5, 5, 5);
			add(opsTable, gbConstraints);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {
			final String request = e.getActionCommand();

			if (request == IOperationsConsole.__BUY) {
				String exchange = _exchange.getText();
				String product = _product.getText();
				String qt = _qt.getText();
				if ("".equals(exchange) || "".equals(product) || "".equals(qt))
					JOptionPane.showMessageDialog(this, __BLANK_FIELD_MSG);
				else
					_trader.buy(exchange, product, qt);

			} else if (request == IOperationsConsole.__SELL) {
				String exchange = _exchange.getText();
				String product = _product.getText();
				String price = _price.getText();
				if ("".equals(exchange) || "".equals(product)
						|| "".equals(price))
					JOptionPane.showMessageDialog(this, __BLANK_FIELD_MSG);
				else
					_trader.sell(exchange, product, price);
			}
		}
	}
}
