package gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;

/**
 * This code is provided for use in the course SWE-622, Distributed Software
 * Systems. Feel free to use and change this code, but the author is not liable
 * for any such use or misuse.
 * 
 * This class implements a table inside a scroll pane for inclusion in JPanel
 * objects. The table logs operations as defined in the course project
 * statement.
 * 
 * @author jpsousa
 * @date Sep 21, 2006
 */
class JScrollOperationsTable extends JScrollPane {
	private static final long serialVersionUID = 1L;

	private OperationsTableModel _operationsTable;

	/**
	 * Creates a scroll pane with a table inside for inclusion in JPanel objects.
	 */
	JScrollOperationsTable() {
		_operationsTable = new OperationsTableModel();
		OperationsTable table = new OperationsTable(_operationsTable);
		table.setPreferredScrollableViewportSize(new Dimension(350, 100));
		setViewportView(table);
	}

	/**
	 * Logs an operation at the bottom of the table
	 * 
	 * @param exchange
	 * @param op
	 *            use the constants declared in IOperationsConsole
	 * @param product
	 * @param qt
	 * @param price
	 * @param stale
	 *            if true, the entry will be logged in red
	 */
	void logOperation(String exchange, String op, String product, String qt,
			String price, boolean stale) {
		String[] opRecord = { exchange, op, product, qt, price };
		_operationsTable.add(opRecord, stale);
	}

	/**
	 * Clears all log entries from the table
	 */
	public void clearLog() {
		_operationsTable.removeAll();
	}

	/**
	 * Overrides the prepareRenderer method in JTable so that stale rows can be
	 * colored
	 * 
	 * @author jpsousa
	 * @date Sep 27, 2006
	 */
	private class OperationsTable extends JTable {
		private static final long serialVersionUID = 1L;

		OperationsTable(OperationsTableModel model) {
			super(model);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.JTable#prepareRenderer(javax.swing.table.TableCellRenderer,
		 *      int, int)
		 */
		public Component prepareRenderer(TableCellRenderer renderer, int row,
				int col) {
			Component c = super.prepareRenderer(renderer, row, col);
			if (_operationsTable.isRowStale(row))
				c.setForeground(Color.red);
			else
				c.setForeground(Color.black);
			return c;
		}
	}

	/**
	 * Implements the data store for the table as an array list
	 * 
	 * @author jpsousa
	 * @date Sep 27, 2006
	 */
	private class OperationsTableModel extends AbstractTableModel {
		private static final long serialVersionUID = 1L;

		private String[] _columnNames = { "Exchange", "Op", "Product", "Qt",
				"Price" };

		private ArrayList<String[]> _data = new ArrayList<String[]>();

		private HashSet<Integer> _staleRows = new HashSet<Integer>();

		/**
		 * Appends an operation record to the end of the table and marks the row
		 * stale, as appropriate
		 * 
		 * @param operation
		 *            in the form {exchange, op, product, qt, price}
		 * @param stale
		 */
		private void add(String[] operation, boolean stale) {
			int row = getRowCount();
			_data.add(operation);
			if (stale)
				_staleRows.add(row);
			fireTableRowsInserted(row, row);
		}

		private boolean isRowStale(int row) {
			return _staleRows.contains(row);
		}

		/**
		 * Clears all records
		 */
		private void removeAll() {
			int lastRow = getRowCount() - 1;
			_data.clear();
			fireTableRowsDeleted(0, lastRow);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getColumnCount()
		 */
		public int getColumnCount() {
			return 5;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getRowCount()
		 */
		public int getRowCount() {
			return _data.size();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getColumnName(int)
		 */
		public String getColumnName(int col) {
			return _columnNames[col];
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		public Object getValueAt(int row, int col) {
			return ((String[]) _data.get(row))[col];
		}

		/*
		 * (non-Javadoc) JTable uses this method to determine the default
		 * renderer/ editor for each cell.
		 * 
		 * @see javax.swing.table.TableModel#getColumnClass(int)
		 */
		public Class<?> getColumnClass(int c) {
			return getValueAt(0, c).getClass();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int,
		 *      int)
		 */
		public void setValueAt(Object value, int row, int col) {
			((String[]) _data.get(row))[col] = value.toString();
			fireTableCellUpdated(row, col);
		}
	}
}
